<?php
// +----------------------------------------------------------------------+
// | Author:  Evgeny Leontev <eleontev@ezactive.com>                      |
// | Copyright (c) 2005 Evgeny Leontev                                    |
// +----------------------------------------------------------------------+
// | This source file is free software; you can redistribute it and/or    |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// |                                                                      |
// | This source file is distributed in the hope that it will be useful,  |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
// | Lesser General Public License for more details.                      |
// +----------------------------------------------------------------------+

require_once 'eleontev/SQL/ModifySql.php'; 


class AppModel extends BaseModel
{

	var $tbl_pref_custom;
	var $tbl_pref;	
	var $tables = array();
	var $custom_tables = array();
	var $db;
	var $limit;
	var $tbl;
	var $sql_params = '1';   					// will be added to select rows
	var $sql_params_ar = array();
	var $sql_params_order = '';   		
	var $delete_mode = 1; // or 2 (1- delete where id IN (1), 2- delete where id IN (1,2,3)) 
	
	
	function AppModel() {

		$reg =& Registry::instance();
		$this->db 		=& $reg->getEntry('db');
		$this->tbl_pref =& $reg->getEntry('tbl_pref');
		$this->limit 	=& $reg->getEntry('limit');
		//$this->priv 	=& $reg->getEntry('priv');
				  
		$this->tbl = $this->setTableNames();
		//echo "<pre>"; print_r($this->tbl); echo "</pre>";	
	}
	
	
	function checkPriv(&$priv, $action) {
		$priv->check($action);
	}
	
	
	function setTableNames() {
		$t = array();
		$t1 = $this->_setTableNames($this->tables, $this->tbl_pref, $this->tbl_pref_custom);
		$t2 = $this->_setTableNames($this->custom_tables, $this->tbl_pref);
		
		return (object) array_merge($t1, $t2);
	}
	
	
	function _setTableNames($tables, $tbl_pref = '', $tbl_pref_custom = '') {
		$tbl =  array(); 
		foreach($tables as $k => $v) {
			$t = (!is_int($k)) ? $k : $v;
			$tbl[$t] = $tbl_pref . $tbl_pref_custom . $v;
		}
		
		return $tbl;
	}
	
	
	// set diffrent sql params used when list records
	function setSqlParams($params, $key = null) {
		static $i = 0;
		$key = ($key) ? $key : $i++;
		
		$this->sql_params_ar['default'] = 1;
		$this->sql_params_ar[$key] = $params;
		$this->sql_params = implode(' ', $this->sql_params_ar);
		
		//echo "<pre>"; print_r($this->sql_params); echo "</pre>";
	}
	
	function setSqlParamsOrder($val) {
		$this->sql_params_order = $val;
	}
	
	
	// could be used in PHP5 for to interact with obj classes
	//function __call($method, $args){
	//	return call_user_func_array(array($this->obj, $method), $args);
	//}
	
	
	// COMMON DB ACTIONS & SQL // ---------------------
	
	function getById($record_id) {
		$sql = "SELECT * FROM {$this->tbl->table} WHERE id = ?";
		$record_id = (int) $record_id;
		$result =& $this->db->Execute($sql, $record_id) or die(db_error($sql));
		return $result->FetchRow();
	}
	
	
	function &getRecords($limit = -1, $offset = -1) {
		$sql = $this->getRecordsSql();
		
		if($limit == -1) {
			$result =& $this->db->Execute($sql) or die(db_error($sql));
		} else {
			$result =& $this->db->SelectLimit($sql, $limit, $offset) or die(db_error($sql));
		}
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//echo "<pre>"; print_r($result); echo "</pre>";
		//echo "<pre>"; print_r($result->GetArray()); echo "</pre>";
		
		return $result->GetArray();
	}

	
	function getCountRecordsSql() {
		return $this->getRecordsSql();
	}
	
	
	function add(&$obj) {
		$sql = ModifySql::getSql('INSERT', $this->tbl->table, $obj->get());
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//exit();
		
		$this->db->Execute($sql) or die(db_error($sql));
		return $this->db->Insert_ID(); 
	}
	
	
	function update(&$obj) {
		$sql = ModifySql::getSql('UPDATE', $this->tbl->table, $obj->get());
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//exit();
		
		$this->db->Execute($sql) or die(db_error($sql));
		return $this->db->Insert_ID();
	}
	
	
	function save(&$obj) {
		$sql = ModifySql::getSql('REPLACE', $this->tbl->table, $obj->get());
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//exit();
		
		$this->db->Execute($sql) or die(db_error($sql));
		return $this->db->Insert_ID();
	}	
	
	
	function status($value, $id) {
		$sql = "UPDATE {$this->tbl->table} SET active='%s' WHERE id IN (%s)";
		$sql = sprintf($sql, mysql_escape_string($value), $this->idToString($id));
		$this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function delete($record_id) {
		
		// convert to string 1,2,3... to use in IN()
		$record_id = $this->idToString($record_id);
		
		$sql = "DELETE FROM {$this->tbl->table} WHERE id IN (%s)";
		$sql = sprintf($sql, $record_id);
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	
	// UTIL TO GENERATE SQL // ---------------------
	
	// convert to string 1,2,3... to use in IN()
	function idToString($val) {
		
		if(is_array($val)) { 
			
			if($this->delete_mode == 2) {
				$val = implode(',', $val); 
			} else {
				exit('WRONG UTIL SQL');
			}
			
		} else { 
			$val = (int) $val; 
		}
		
		return $val;
	}
}
?>