<?php
// +----------------------------------------------------------------------+
// | Author:  Evgeny Leontev <eleontev@ezactive.com>                      |
// | Copyright (c) 2005 Evgeny Leontev                                    |
// +----------------------------------------------------------------------+
// | This source file is free software; you can redistribute it and/or    |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// |                                                                      |
// | This source file is distributed in the hope that it will be useful,  |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
// | Lesser General Public License for more details.                      |
// +----------------------------------------------------------------------+

/**
 * Uploader is a class used to manage uploading.
 * 
 * @since 20/05/2003
 * @author Evgeny Leontev <info@ezactive.com>
 * @access public
 */

/*
UPLOAD_ERR_OK
Value: 0; There is no error, the file uploaded with success. 

UPLOAD_ERR_INI_SIZE
Value: 1; The uploaded file exceeds the upload_max_filesize directive in php.ini. 

UPLOAD_ERR_FORM_SIZE
Value: 2; The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the html form. 

UPLOAD_ERR_PARTIAL
Value: 3; The uploaded file was only partially uploaded. 

UPLOAD_ERR_NO_FILE
Value: 4; No file was uploaded. 
*/

class Uploader
{

	var $allowed_type 		= array();
	var $denied_type	= array();
	
	var $allowed_extension = array();	
	var $denied_extension = array();		
	
	var $max_file_size 		= array(500);		// max file size in kb (1kb = 1024b)
	var $not_uploaded 		= array();
	var $dest_dir;
	var $suffix;
	var $error = false;

	// set permitted_ext for upload
	function Uploader() {
	}
	
	
	// set suffix for reason unique
	function setSuffix($num) {
		$this->suffix = '_' . preg_replace("/(.)/e","chr(mt_rand(ord(0),ord(9)))",str_repeat('.',$num));
	}
	
	
	// set allowed type for upload example: 'application/pdf', 'application/msword'
	function setAllowedType($type) {
		$type = (is_array($type)) ? $type : func_get_args();
		foreach($type  as $k => $v) {
			$this->allowed_type[] = $v;
		}
	}
	

	function setDeniedType($type) {
		$type = (is_array($type)) ? $type : func_get_args();
		foreach($type  as $k => $v) {
			$this->denied_type[] = $v;
		}
	}
	
	
	function setAllowedExtension($extension) {
		$type = (is_array($type)) ? $type : func_get_args();
		foreach($type  as $k => $v) {
			$this->allowed_extension[] = $v;
		}
	}	
	
	
	function setDeniedExtension($extension) {
		$type = (is_array($type)) ? $type : func_get_args();
		foreach($type  as $k => $v) {
			$this->denied_extension[] = $v;
		}
	}		
	
	
	// set permitted size for upload
	// can specified different sizes for diiferent types
	function setMaxSize($size, $ext = 0) {
		$this->max_file_size[$ext] = $size;
	}
	
	
	
	// return true if file size not bigger
	function maxFileSizeCheck($type, $size, $error) {
		
		if(isset($this->max_file_size[$type])) {
			$alloved_size = $this->max_file_size[$type]*1024;
		} else { 
			$alloved_size = $this->max_file_size[0]*1024;
		}
		
		//1 - The uploaded file exceeds the upload_max_filesize directive in php.ini. 
		//2 - The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the html form. 
		if($error == 1 || $error == 2) {
			return false;
		} elseif ($size > $alloved_size) {
			return false;
		}
		
		return true;
	}
	
	
	function setUploadedDir($dir) {
		if(!is_dir($dir)) { mkdir($dir, 0755);  } // create dir and aplly permission
		else 			  { chmod ($dir, 0755); } // just aplly permission (just in case)
		
		$this->dest_dir = $dir;
	}
	
	
	
	// $files should be like $_FILES or maybe $_FILES['some_name']
	// in form field should be specified as simple name 'image' not as 'image[]'
	function upload($files) {
		
		$already_uploaded = array();
		
		foreach($files as $k => $v) {			
			
			$v['extension'] = substr($v['name'], strrpos($v['name'], ".")+1);
			
			//check for allowed type
			if(!is_writable($this->dest_dir)){
				$uploaded['bad']['not_writetable_dir'] = $this->dest_dir;
				break;			
			
			// if file uploaded already if multiple upload
			} elseif(!$v['name'] || in_array($v['name'], $already_uploaded)) {
				continue;
			
			//check for allowed type
			} elseif($this->allowed_type && !in_array($v['type'], $this->allowed_type)){
				$uploaded['bad']['wrong_type'][$v['name']] = $v['type'];
				continue;
			
			//check for not allowed type
			} elseif($this->denied_type && in_array($v['type'], $this->denied_type)){
				$uploaded['bad']['wrong_type'][$v['name']] = $v['type'];
				continue;			
			
			//check for allowed extension
			} elseif($this->allowed_extension && !in_array($v['extension'], $this->allowed_extension)){
				$uploaded['bad']['wrong_extension'][$v['name']] = $v['extension'];
				continue;
			
			//check for not allowed extension
			} elseif($this->denied_extension && in_array($v['extension'], $this->denied_extension)){
				$uploaded['bad']['wrong_extension'][$v['name']] = $v['extension'];
				continue;				
			
			// too big image size
			} elseif(!$this->maxFileSizeCheck($v['type'], $v['size'], $v['error'])) {
				$uploaded['bad']['big_size'][$v['name']] = $this->myfileSize($v['size']);
				continue;
			
			// The uploaded file was only partially uploaded. 
			} elseif($v['error'] == 3) {
				$uploaded['bad']['partially'][$v['name']] = $v['size'];
				continue;
			
			// No file was uploaded. 
			} elseif($v['error'] == 4) {
				$uploaded['bad']['not_known'][$v['name']] = $v['none'];
				continue;
			
			} else {		
			
				if($this->suffix) {
					$ext = substr($v['name'] ,strrpos($v['name'],"."));
					$name = substr($v['name'] ,0 ,strpos($v['name'],"."));
					$v['name'] = $name . $this->suffix . $ext;
				}
				
				$name = $this->nameToSafe($v['name']);
				$file = $this->dest_dir . '/' . $name;
				$move = move_uploaded_file($v['tmp_name'], $file);
				
				if(@!$move) {
					$this->error = true;
					$uploaded['bad']['not_known'][$v['name']] = 'none'; // not uploaded img array
				} else {
					chmod ($file, 0644);  // octal; correct value of mode
					$key = substr($k, strrpos($k, "_")+1);
					$uploaded['good'][$key] = $v;
														 
					$already_uploaded[] = $v['name']; // not to upload twice
				}	
			}
		}
		
		//echo "<pre>"; print_r($uploaded); echo "</pre>";
		//$not_uploaded_s = serialize($not_uploaded_s);
		return $uploaded;
	}

	
/*
	function errorBox($tmpl, $arr, $msg) {
		
		//if(!class_exists('tplTemplatez')) { return; };

		$tpl = new tplTemplatez($tmpl);
			
		foreach($arr as $k => $v) {
			foreach($v as $k1 => $v1) {
				
				$array['error_type'] = $msg[$k]; // from upload msg	
				$array['file_name'] = $k1;
				$array['value'] = $v1;
				
				$tpl->tplParse($array, 'row');
			}
		}
		
		$tpl->tplParse($msg);
		return $tpl->tplPrint(1);
	}
*/
	
	
	function errorBox($errors, $msg = array()) {
		
		$msg = ($msg) ? $msg : AppMsg::getErrorMsgs();
		
		if(isset($errors['not_writetable_dir'])) {
			
			$msg_error = array('title' => $msg['error_title_msg'],
			                   'body'  => $msg['not_writetable_dir_msg']);
		
			return BoxMsg::factory('error', $msg_error);
		}
		
		
		$html = array();
		$html[] = '<table width="100%" cellspacing="2" cellpadding="0" border="0">';		
		
		$html[] = '<tr>';		
		$html[] = '<td><b>' . $msg['file_name_msg'] . '</b></td>';
		$html[] = '<td><b>' . $msg['error_type_msg'] . '</b></td>';		
		$html[] = '<td><b>' . $msg['value_msg'] . '</b></td>';
		$html[] = '</tr>';
		
		foreach($errors as $k => $v) {
			foreach($v as $k1 => $v1) {
							
				$html[] = '<tr><td colspan="3" bgcolor="#808080"></td></tr>';	
				$html[] = '<tr>';
				$html[] = '<td>'.$k1.'</td>';
				$html[] = '<td>'.$msg[$k.'_msg'].'</td>';
				$html[] = '<td>'.$v1.'</td>';
				$html[] = '</tr>';
			}
		}
		
		//$html[] = '<tr><td colspan="3" bgcolor="#808080"></td></tr>';
		$html[] = '</table>';
		
		$msg_error = array('title'  => $msg['error_title_msg'],
		                   'title' => $msg['uploaded_error_msg'], 
						   'body'   => implode("\n", $html));
		
		return BoxMsg::factory('error', $msg_error);
	}	
	
	
   /**
    * Format a file name to be safe
    *
    * @param    string $file   The string file name
    * @param    int    $maxlen Maximun permited string lenght
    * @return   string Formatted file name
    * @see HTTP_Upload_File::setName()
    */
    function nameToSafe($name, $maxlen=250) {
        $noalpha = '@';
        $alpha =   'aeiouaeiouaeiouAEIOUAEIOUAEIOUaeiouAEIOUncCa';
        $name = substr ($name, 0, $maxlen);
        $name = strtr ($name, $noalpha, $alpha);
        // not permitted chars are replaced with "_"
        return ereg_replace ('[^a-zA-Z0-9,._\+\()\-]', '_', $name);
    }
	
	
	function myfileSize($file){
		
		if(is_numeric($file))
			$file_size = $file+0;
		else
			$file_size = filesize($file);
		
		
		if ($file_size >= 1073741824)
			$file_size = round($file_size / 1073741824 * 100) / 100 ." gb"; 
			
		elseif ($file_size >= 1048576)
			$file_size = round($file_size / 1048576 * 100) / 100 ." mb"; 
			
		elseif ($file_size >= 1024)
			$file_size = round($file_size / 1024 * 100,-2) / 100 ." kb"; 
			
		else 
			$file_size = $file_size . " b"; 
		
		return $file_size;
	}

}


/*
// images types
"image/bmp" => ".bmp, .ico",
"image/x-pcx" => ".pcx",
"image/gif" => ".gif",
"image/tiff" => ".tiff .tif .img",
"image/tif" => ".tiff .tif .img",
"image/pjpeg" => ".jpg, .jpeg",
"image/jpeg" => ".jpg, .jpeg",
"image/x-jpeg" => ".jpg, .jpeg",

// archive file types
"application/x-gzip-compressed"	=> ".tar.gz, .tgz",
"application/x-zip-compressed"	=> ".zip",
"application/x-tar" => ".tar",
"application/x-gtar" => ".gtar",
"application/zip" => ".zip",

// text types
"text/plain" => ".html, .php, .txt, .inc (etc)",
"text/richtext" => " (etc)",

// multimedia types
"application/x-shockwave-flash" => ".swf",
"application/futuresplash" => ".spl",
"audio/midi" => ".mid, .midi",
"audio/wav" => ".wav",
"audio/basic" => ".au, .snd",
"audio/x-pn-realaudio" => ".ra, .ram, .rm",
"audio/x-aiff" => ".aiff, .aifc",
"video/mpeg" => ".mpeg, .mpg",
"video/quicktime" => ".qt, .mov",
"video/x-msvideo" => ".avi",


// executable types
"application/octet-stream" => ".exe, .fla (etc)",

// document types
"application/pdf" => ".pdf",
"application/x-pdf" => ".pdf",
"application/postscript" => ".ps, .eps (etc)",
"application/ms-powerpoint" => ".ppt, .pot, pps, (etc)",
"application/msword" => ".doc",
"application/vnd.ms-excel" => ".xls"
*/

//echo Uploader::myfileSize(1070808);
?>