<?php
// +----------------------------------------------------------------------+
// | Author:  Evgeny Leontev <eleontev@ezactive.com>                      |
// | Copyright (c) 2005 Evgeny Leontev                                    |
// +----------------------------------------------------------------------+
// | This source file is free software; you can redistribute it and/or    |
// | modify it under the terms of the GNU Lesser General Public           |
// | License as published by the Free Software Foundation; either         |
// | version 2.1 of the License, or (at your option) any later version.   |
// |                                                                      |
// | This source file is distributed in the hope that it will be useful,  |
// | but WITHOUT ANY WARRANTY; without even the implied warranty of       |
// | MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU    |
// | Lesser General Public License for more details.                      |
// +----------------------------------------------------------------------+

// some common functions
class ELCommon 
{
	var $form_action;
	var $form_submit_field;  // to know about sertain form submit
	
	function setFormAction($form_action) {
		$this->form_action =& $form_action;
	}
	
	function setFormSubmitField($form_field) {
		$this->form_submit_field =& $form_field;
	}
	
	//function setClassVar($var, $value) {
	//	$this->$var = $value;
	//}
	
}


/**
 * SearchParams is a class to generate "search string" from form submition.
 *
 * @version 1.0
 * @since 11/07/2003
 * @author Evgeny Leontev 
 * @access public
 */
class SearchParams extends ELCommon
{	
	
	var $filter;
	var $sess_name = false;
	var $default_sql = '1=1';
	var $default_var = 'all';
	var $sql_sign = '=';
	var $sql_operator = 'AND';
	var $sql_group = false;
	var $sql = false;
	
	// for current field
	var $def_var = false;
	var $def_sql = false;
	var $dateBetweenDefaults = false;
	var $full_text_search = false;
	
	function SearchParams() {
		return true;
	}
	
	// set session name if need to remember submited data 
	function setSessionName($session_name) {
		$this->sess_name = $session_name;
	}
	
	// =, LIKE, ...
	function setSqlSign($sign) {
		$this->sql_sign = $sign;
	}
	
	// AND, OR, ...
	function setSqlOperator($operator) {
		$this->sql_operator = $operator;
	}
	
	// replace * to %, for use with LIKE in sql query
	// serach_all means we add a % to end of str
	function setLikeSql($str, $search_all = false) {
		$str = str_replace('*', '%', $str);
		$this->setSqlSign('LIKE');
		return ($search_all && $str) ? $str . '%' : $str;
	}
	
	
	function parseSeachString($str, $min_letters = 2) {
		
		//     .
		$str = substr($str, 0, 64);
		
		//     "" .
		$str = preg_replace("/[^\w\x7F-\xFF\s]/", " ", $str);
		
		
		if($min_letters) {
		
			// ,      
			//    -   ,    . 
			//,     ,     
			//(  ,   "{1,2}"  "{1, - }").
			
			$preg = sprintf("/\s(\S{1,%d})\s/", $min_letters);
			$str = trim(preg_replace($preg, " ", ereg_replace(" +", "  "," $str ")));
			
			//     -     
			//(        ).
			$str = ereg_replace(" +", " ", $str);
		}
		
		return $str;
	}
	
	
	
	// make full text search
	function setFullTextSearch() {
		$this->full_text_search = true;
	}
	
	
	//function setMoreTables($table, $where_sql) {
	//	$this->more_tables[] = 
	//}
	
	// set sql if could not do in normal way
	function setCustomSql($value, $form_field, $sql, $operator = false) {
		
		if($this->form_submit_field) {
			if($this->sess_name != false) {
				$_SESSION[$this->sess_name][$form_field . '_var'] = $value;
				$_SESSION[$this->sess_name][$form_field . '_sql'] = $sql;
			}
		} else {
		
			// if has such session
			if(isset($_SESSION[$this->sess_name][$form_field . '_var'])) {
				$sql = $_SESSION[$this->sess_name][$form_field . '_sql'];
				$value = $_SESSION[$this->sess_name][$form_field . '_var'];
			}
		}
		
		$this->filter[$form_field]['var'] = $value;
		$this->filter[$form_field]['sql'] = $sql;
		$this->filter[$form_field]['operator'] = ($operator) ? $operator : $this->sql_operator;
		$this->filter[$form_field]['group'] = '';
	}
	
	
	/**
	* setDefaults -- set defaults values if no form submition
	*
	* @param    string    $def_var       default value
	* @param    string    $def_sql       (optional) default sql string to use query
	*
	* @return   void
	* @access   public    
	*/
	function setDefaults($def_var, $def_sql = false) {
		$this->def_var = $def_var;
		$this->def_sql = $def_sql;
	}
	
	
	
   /**
	* setParams -- generate array wit paams per one item to use in sql query
	*
	* @param    string    $value         value of submited form field
	* @param    string    $form_field    field name of submited form field
	* @param    mixed     $sql_field     sql field name
	* @param    string    $group         (optional) 'start' or 'stop' 
	* @param    string    $operator      (optional) sql operator to use in sql string, AND by default
	*
	* @example  $obj->setParams($_GET['blah'], 'blah', 'p.blah', '', 'OR');
	*
	* @return   void
	* @access   public    
	*/
	function setParams($value, $form_field, $sql_field, $group = false, $operator = false) {
		
		// if is array $sql_field can be in full text search
		if(is_array($sql_field)) { $sql_field = implode(', ', $sql_field); }
		
		// for using in IN
		if(is_array($value)) { $value = '(' . implode(',', $value) . ')'; }
		
		// if form submited and value not default
		if($this->form_submit_field) {
			$ret = mysql_escape_string(trim($value));
			$sql = ($this->full_text_search) ? "MATCH ($sql_field) AGAINST ('{$ret}')" : "{$sql_field} {$this->sql_sign} '{$ret}'";
			$var = trim($value);
			
			// set session if needed
			if($this->sess_name != false) {
				$_SESSION[$this->sess_name][$form_field . '_var'] = $var;
				$_SESSION[$this->sess_name][$form_field . '_sql'] = $sql;
			}
			
		} else {
			
			// if has such session
			if(isset($_SESSION[$this->sess_name][$form_field . '_var'])) {
				$sql = $_SESSION[$this->sess_name][$form_field . '_sql'];
				$var = $_SESSION[$this->sess_name][$form_field . '_var'];
			
			// by default
			} elseif($this->def_var !== false) {	
				if($this->full_text_search) {
					$sql = ($this->def_sql) ? $this->def_sql : "MATCH ($sql_field) AGAINST  ('{$this->def_var}')";
				} else {
					$sql = ($this->def_sql) ? $this->def_sql : "{$sql_field} {$this->sql_sign} '{$this->def_var}'";
				}
				$var = $this->def_var;
			
			// all others
			} else {
				$sql = $this->default_sql;
				$var = $this->default_var;
			}
		}
		
		// to strip IN'(1,2)' to get IN (1,2)
		$sql = str_replace(array("'(", ")'"), array("(", ")"), $sql);
		
		$this->filter[$form_field]['var'] = $var;
		$this->filter[$form_field]['sql'] = $sql;
		$this->filter[$form_field]['operator'] = ($operator) ? $operator : $this->sql_operator;
		$this->filter[$form_field]['group'] = $group;
		
		// unset defaults for current var & sql
		$this->def_var = false;
		$this->def_sql = false;
	}
	
	
   /**
	* getVars -- for use in form field if needed
	*
	* @return   array
	* @access   public    
	*/
	function getVars() {
		$data = array();
		foreach($this->filter AS $k => $v) {
			
			if(is_array($v['var'])) {
				foreach($v['var'] as $k1 => $v1) {
					if($v1 && $v1 !== $this->default_var) {
						$data[$k1] = str_replace('%', '*', $v1); // convert back
					}
				}
			} else {
				if($v['var'] && $v['var'] !== $this->default_var) {
					$data[$k] = str_replace('%', '*', $v['var']); // convert back
				}
			} 
		}
		return $data;
	}
	

   /**
	* getSQL -- generate sql string, for use in sql query
	*
	* @return   string
	* @access   public    
	*/
	function getSQL() {
		$sql_str = false;
		foreach($this->filter AS $k => $v) {
			//if($v['sql'] != $this->default_sql) {
			if($v['var'] !== '' && $v['var'] !== $this->default_var) {
				if($v['group']) {
					$sql_str .= ($v['group'] == 'start') ? $v['operator'].' ('.$v['sql'] : $v['operator'].' '.$v['sql'].')';
					$sql_str .=  "\n";
				} else {
					$sql_str .= $v['operator'] . ' ' . $v['sql'] . "\n";
				}
			}
		}
		$sql_str = $this->default_sql . ' ' . $sql_str;
		return $sql_str;
	}
	
	
	// $date_from_val, $date_till_val - in UNIXTIMESTAMP
	function setDateBetweenDefaults($date_from_val, $date_till_val) {
		$this->def_date_from = $date_from_val;
		$this->def_date_till = $date_till_val;		
		$this->dateBetweenDefaults = true;
	}
	
	
	// date BETWEEN '20031210' AND '20031215'";
	function setDateBetweenParams($date_from_val, $date_till_val, $sql_field, $group = false, $operator = false) {
		
		//if(!empty($date_from_val)) {
		if($this->form_submit_field) {	
		
			if($date_from_val) {
				$date_from_sql = date('Ymd', $date_from_val); // 20031203
				$date_till_sql = date('Ymd', $date_till_val);
				$sql = "{$sql_field} BETWEEN '{$date_from_sql}' AND '{$date_till_sql}'";
				$var_from = $date_from_val;
				$var_till = $date_till_val;
			} else {
				$var_from = $this->default_var;
				$var_till = $this->default_var;
				$sql = $this->default_sql;
			}
			
			
			// set session if needed
			if($this->sess_name != false) {
				$_SESSION[$this->sess_name]['date_beetwen_from'] = $var_from;
				$_SESSION[$this->sess_name]['date_beetwen_till'] = $var_till;
				$_SESSION[$this->sess_name]['date_beetwen_sql']  = $sql;
			}
			
		} else {
			
			// if has such session
			if(isset($_SESSION[$this->sess_name]['date_beetwen_from'])) {
				$var_from = $_SESSION[$this->sess_name]['date_beetwen_from'];
				$var_till = $_SESSION[$this->sess_name]['date_beetwen_till'];
				$sql = $_SESSION[$this->sess_name]['date_beetwen_sql'];
			
			// by default
			} elseif($this->dateBetweenDefaults) {
				$var_from = $this->def_date_from;
				$var_till = $this->def_date_till;
				
				$date_from_sql = date('Ymd', $var_from); // 20031203
				$date_till_sql = date('Ymd', $var_till);
				$sql = "{$sql_field} BETWEEN '{$date_from_sql}' AND '{$date_till_sql}'";
			
			// all others
			} else {
				$var_from = $this->default_var;
				$var_till = $this->default_var;
				$sql = $this->default_sql;
			}
		}
		
		$this->filter['date_beetwen']['var']['from'] = $var_from;
		$this->filter['date_beetwen']['var']['till'] = $var_till;
		$this->filter['date_beetwen']['sql'] = $sql;
		$this->filter['date_beetwen']['operator'] = ($operator) ? $operator : $this->sql_operator;
		$this->filter['date_beetwen']['group'] = $group;
		
		// unset defaults for current var & sql
		//$this->def_var = false;
		//$this->def_sql = false;
	}	

} // <-- class end



/*
     .

$search = substr($search, 0, 64);

64      . 
     "" .

$search = preg_replace("/[^\w\x7F-\xFF\s]/", " ", $search);

 ,      
    -   ,    . 
,     ,     
(  ,   "{1,2}"  "{1, - }").

$good = trim(preg_replace("/\s(\S{1,2})\s/", " ", ereg_replace(" +", "  "," $search ")));

     -     
(        ).

$good = ereg_eplace(" +", " ", $good);



SELECT id, question, MATCH (question, answer) AGAINST (
'license') AS score
FROM s_kb_entry WHERE MATCH (question, answer) AGAINST
('license');

SHOW VARIABLES LIKE 'ft%';

SELECT *
FROM s_kb_entry WHERE @a := CONCAT( artist, album )
LIKE '%Prodigy%' OR @a
LIKE '%Breathe%'

SELECT title, date_format(material_date,'%e.%c.%y') AS date1, IF(text like '%word1 word2 word3%', 3*10, 0) + IF(text LIKE '%word1%', 9, 0) + IF(text LIKE '%word2%', 9, 0) + IF(text LIKE '%word3%', 9, 0) AS relevance FROM table WHERE text LIKE '%word1%' OR text LIKE '%word2%' OR text LIKE '%word3%' ORDER BY relevance DESC, material_date DESC

 ,       MySQL.        :

SELECT title, date_format(material_date,'%e.%c.%y') AS date1, MATCH text AGAINST('word1 word2 word3') AS relevance FROM table WHERE text LIKE '%word1%' OR text LIKE '%word2%' OR text LIKE '%word3%' ORDER BY relevance DESC, material_date DESC


*/
?>