<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once 'eleontev/HTML/FormSelect.php';
 
class SettingModel extends AppModel
{

	var $tables = array('table'=>'setting', 'setting', 'setting_to_value', 'setting_input', 'priv_module');
	
	var $module_id;
	var $module_name;
	
	
	function &getSettings() {
		
		$sql = "
		SELECT 
			s.setting_key,
			s.input_id,
			IFNULL(sv.setting_value, s.default_value) AS value
		FROM 
			s_setting s
		LEFT JOIN s_setting_to_value sv ON s.id = sv.setting_id
		WHERE 1
			AND s.active = 1 
			AND s.module_id = %d";
		
		$sql = sprintf($sql, $this->module_id);
		$result = &$this->db->Execute($sql) or die(db_error($sql));
		$row = &$result->GetArray();
		
		foreach($row as $k => $v) {
			if($v['input_id'] == 1 && strpos(',', $v['value'])) {
				$data[$v['setting_key']] = $this->_valueToArray($v['value']);
			} else {
				$data[$v['setting_key']] = $v['value'];
			}
		}
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		//echo "<pre>"; print_r($data); echo "</pre>";		
		unset($row);
		return $data;
	}
	
	
	function getSetting($setting_key) {
		
		$sql = "SELECT 
		s.input,
		IFNULL(sv.setting_value, s.default_value) AS value,
		FROM s_setting s,
		LEFT JOIN s_setting_to_value sv ON s.id = sv.setting_id
		WHERE s.active = 1 
		AND s.module_id = %d
		AND s.setting_key = '%s'";
		
		$sql = sprintf($sql, $this->module_id, $setting_key);
		$result = &$this->db->Execute($sql) or die(db_error($sql));
		$row = &$result->GetRow($sql);
		
		if($row['input'] == 'select') {
			$row['value'] = $this->_valueToArray($row['value']);
		}
		
		return $row['value'];
	}
		
	
	function getRecordsSql() {
		
		$sql = "
		SELECT 
			s.id AS name,
			s.setting_key,	
			s.messure,
			s.options,
			s.range,
			IFNULL(sv.setting_value, s.default_value) AS value,
			si.input
		FROM 
			{$this->tbl->setting} s,
			{$this->tbl->setting_input} si
		
		LEFT JOIN {$this->tbl->setting_to_value} sv ON s.id = sv.setting_id
		
		WHERE s.active = 1 
		AND s.module_id = ?
		AND s.input_id = si.id
		ORDER BY s.sort_order";
		
		return $sql;
	}
	
	
	
	function &getRecords() {
		$sql = $this->getRecordsSql();
		$result =& $this->db->Execute($sql, $this->module_id) or die(db_error($sql));
		while($row = $result->FetchRow()){
			
			
			if($row['input'] == 'select') {
				$row['value'] = $this->_valueToArray($row['value']);
			}
			
			if($row['range']) {
				if(strpos($row['range'], '-')) {
					$row['range'] = $this->_valueToOtherArray($row['range']);							
				} else {
					$row['range'] = $this->_valueToArray($row['range']);				
				}

			}			
			
			$data[$row['setting_key']] = $row;
		}
		
		return $data;
	}
	
	
	function getSettingKeys() {
		$sql = "SELECT s.id, s.setting_key
		FROM {$this->tbl->setting} s
		WHERE s.active = 1 
		AND s.module_id = ?";
		
		$result =& $this->db->Execute($sql, $this->module_id) or die(db_error($sql));
		return  $result->GetAssoc();
	}
	
	
	function formDataToObj($values) {
		
		$keys = $this->getSettingKeys();
		foreach($values as $k => $v) {
			$values1[$keys[$k]] = $v;
		}
		
		return $values1;
	}
	
	
	function save($data) {
		
		require_once 'eleontev/SQL/MultiInsert.php';
		
		foreach($data as $k => $v) {
			$data[$k] = array($k, $v);
		}
		
		
		$ins = &new MultiInsert;
		$ins->setFields('setting_id', 'setting_value');
		$ins->setValues($data);
		$sql = $ins->getSql($this->tbl->setting_to_value, 'REPLACE');
		$this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function setDefaultValues() {
		
		//$sql = 'DELETE FROM {$this->tbl->setting_to_value} WHERE setting_id = %d';
		$sql = "REPLACE {$this->tbl->setting_to_value} (setting_id, setting_value) 
		SELECT id, default_value 
		FROM {$this->tbl->setting} 
		WHERE module_id = %d";
		$sql = sprintf($sql, $this->module_id);
		$this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function setModuleId($module_name) {
		$sql = "SELECT id FROM {$this->tbl->priv_module} WHERE module_name = '%s'";
		$sql = sprintf($sql, $module_name);
		$result = &$this->db->Execute($sql) or die(db_error($sql));
		
		$this->module_id = $result->Fields('id');
		$this->module_name = $module_name;
	}
	
	
	function _valueToArray(&$val) {
		
		$val = explode(',', $val);
		
		foreach($val as $v) {
			$new_ar[trim($v)] = $v;
		}
		return $new_ar;
	}
	
	
	function _valueToOtherArray(&$val) {
	
		$val = explode(',', $val);
		
		foreach($val as $k => $v) {
			$val[trim($k)] = explode('-', $v);
		}
		
		foreach($val as $k => $v) {
			$arr1[$val[$k][0]] = $val[$k][1];
		}
		
		return $arr1;
	}
}
?>
 