<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once APP_MODULE_DIR . 'knowledgebase/category/inc/KBCategoryModel.php';
require_once APP_MODULE_DIR . 'setting/inc/SettingModel.php';


class KBClientModel extends AppModel
{
	
	var $tbl_pref_custom = 'kb_';
	var $tables = array('entry', 'category', 'entry_to_category', 'comment', 'rating', 
	                    'glossary', 'entry_by_user', 'attachment_to_entry');
						
	var $custom_tables = array('user'=>'member', 'file_entry');
	
	
	var $is_registered = false;
	var $user_id;
	
	var $category_simple_arr = array();
	var $search_params;

	
	// for select article
	var $sql_params = 1;
	var $sql_params_order = '';
	var $sql_params_select = 1;
	var $sql_params_ar = array();
	
	
	// sessions name
	var $session_vote_name = 'kb_vote_';
	var $session_view_name = 'kb_view_';
	
	
	
	
	// set diffrent sql params used when list records
	function setSqlParams($params, $key = null) {
		static $i = 0;
		$key = ($key) ? $key : $i++;
		
		$this->sql_params_ar['default'] = 1;
		$this->sql_params_ar[$key] = $params;
		$this->sql_params = implode(' ', $this->sql_params_ar);
	}
	
	
	// ATTACMENT // ----------------------
	
	function getAttachmentList($entry_id) {
		
/*
		$sql = "SELECT f.* 
		FROM 
			{$this->tbl->file_entry} f,
			{$this->tbl->attachment_to_entry} ae
		WHERE 1
			AND f.id = ae.attachment_id
			AND ae.entry_id = ?";
		
		$data = array($entry_id);
		$result = &$this->db->Execute($sql, $data) or die(db_error($sql));
		return $result->GetAssoc();
*/
	}
	
	
	// SETTINGS // ------------------------	
	
	function &getSettings($module_id) {
		$s = &new SettingModel();
		$s->module_id = $module_id;
		return $s->getSettings();
	}
	
	
	function getSetting($setting_key) {
		return @$this->setting[$setting_key];
	}
	
	
	// USER // ------------------------
	
	function getUserInfo($user_id) {
		require_once APP_MODULE_DIR . 'member/inc/MemberModel.php';
		$u = new MemberModel;
		return $u->getById($user_id);
	}
	
	
	function setUserActive($user_id) {
		$sql = "UPDATE {$this->tbl->user} SET active = 1 WHERE id = '$user_id'";
		return $this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function isUser($confirm_str, $user_id = false) {
		$sql = "SELECT id FROM {$this->tbl->user} WHERE MD5(email) = '$confirm_str'";
		$result =& $this->db->SelectLimit($sql, 1) or die(db_error($sql));
		return $result->Fields('id');
	}
	
	// to know do we need to user registration or not
	// it is used in some plases
	function isUseRegistration() {
			
		return ($this->getSetting('allow_comments')   == 2 || 
		        $this->getSetting('allow_post_entry') == 2);
	}
	
	
	// CATEGORIES // ------------------------
	
	function _getCategoryListSql($parent_id) {
	
		$private_sql = '1';
		$sort = $this->getSetting('category_sort_order');
		
		$sql = "
		SELECT 
			c.*,
			COUNT(DISTINCT cp.id) AS num_subcategories,
			COUNT(DISTINCT e.id) AS num_entries
			#IF(SUM(e.active), SUM(e.active), 0) as num_entries  
		FROM 
			{$this->tbl->category} c
		LEFT JOIN {$this->tbl->entry_to_category} ec ON c.id = ec.category_id
		LEFT JOIN {$this->tbl->entry} e ON e.id = ec.entry_id AND e.active = 1
		LEFT JOIN {$this->tbl->category} cp ON c.id = cp.parent_id
		
		WHERE 1
			AND c.parent_id = '{$parent_id}' 
			AND c.active = 1
			AND {$private_sql}
		GROUP BY c.id
		ORDER BY {$sort}";
		
		return $sql;
	}
	
	
	function &getCategoryList($parent_id = 0) {
		
		$data = array();
		$sql = $this->_getCategoryListSql($parent_id);
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		while($row = $result->FetchRow()){
			//if($row['num_entries'] == 0 && $row['num_subcategories'] == 0) { continue; }
			$data[$row['id']] = $row;
		}

		return $data;
	}
	
	
	// return true if category is private and we use it
	function isCategoryPrivate($category_id) {
		$sql = "SELECT private FROM {$this->tbl->category} WHERE id = ?";
		$result =& $this->db->Execute($sql, $category_id) or die(db_error($sql));
		return $result->Fields('private');
	}
	
	
	function getCategoryModel() {
		return new KBCategoryModel;
	}
	
	
	function getCategoryName($category_id) {
		$c = $this->getCategoryModel();
		$row = $c->getById($category_id);
		return $row['name'];
	}
	
	
	// ARTICLES // ------------------------	
	
	function getEntriesSql($body_words = 70) {
		
		//$private_sql = $this->_getEntryPrivateSqlParams();
		$body_words_sql = ($body_words != 'all') 
						  ? "SUBSTRING_INDEX(e.body,' ',$body_words)" 
						  : 'body'; 
		
		
		$sql = "
		SELECT 
			e.*,
			{$body_words_sql} AS body, 
			cat.id AS category_id,
			cat.name AS category_name,
			cat.private AS category_private,
			cat.commentable AS commentable,
			cat.ratingable,
			
			r.votes AS votes,
			(r.rate/r.votes) AS rating,
			#IF(c.active = 1, COUNT(DISTINCT c.id), 0) AS comment_num,
			IFNULL(SUM(c.active), 0) AS comment_num,
			
			UNIX_TIMESTAMP(e.date_updated) AS ts_updated,
			{$this->sql_params_select}
		
		FROM 
			{$this->tbl->entry} e, 
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat
		
		LEFT JOIN {$this->tbl->comment} c ON e.id = c.entry_id
		LEFT JOIN {$this->tbl->rating} r ON e.id = r.entry_id
		
		WHERE 1
			AND e.active = 1
			AND cat.active = 1
			AND e.id = e_to_cat.entry_id
			AND cat.id = e_to_cat.category_id
			AND {$this->sql_params}
			GROUP BY e.id
		{$this->sql_params_order}";
		
		//echo "<pre>"; print_r($sql); echo "</pre>";
		return $sql;
	}		
	
	
	function getEntryCountSql() {
		
		$sql = "SELECT COUNT(*)
		FROM 
			{$this->tbl->entry} e, 
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat
		WHERE 
			e.active = 1
			AND cat.active = 1
			AND e.id = e_to_cat.entry_id
			AND cat.id = e_to_cat.category_id
			AND {$this->sql_params}";
		
		return $sql;
	}
	
	
	function getEntriesRelated($category_id, $entry_id) {
		
		$sql = "SELECT e.id, e.title 
		FROM 
			{$this->tbl->entry} e, 
			{$this->tbl->category} cat,
			{$this->tbl->entry_to_category} e_to_cat
		
		WHERE e.active = 1
			AND cat.active = 1
			AND e.id = e_to_cat.entry_id
			AND cat.id = e_to_cat.category_id 
			AND cat.id = %d
			AND e.id != %d
		GROUP BY e.id, cat.id";
		
		$sql = sprintf($sql, $category_id, $entry_id);
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetArray();
	}
	
	
	function &getEntryList($limit, $offset, $body_words = 70) {	

		$sql = $this->getEntriesSql($body_words);
		$result =& $this->db->SelectLimit($sql, $limit, $offset) or die(db_error($sql));
		return $result->GetArray();
	}
	
	
	function &getEntryById($entry_id, $category_id) {
		
		$sql = sprintf("AND e.id = %d AND cat.id = %d", $entry_id, $category_id);
		$this->setSqlParams($sql);
				   
		$sql = $this->getEntriesSql('all');
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->FetchRow();
	}
	
	
	function getEntryTitle($entry_id) {
		$sql = "SELECT title FROM {$this->tbl->entry} WHERE id = %d";
		$sql = sprintf($sql, $entry_id);
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->Fields('title');		
	}
	
	
	function _getEntryPrivateSqlParams() {
		$private_sql = 'AND 1';
		if($this->is_registered != true) {
			$private_sql = 'AND cat.private = 0';
		}
		
		return $private_sql;
	}
	

/*
	function getSearchParams() {
		
		require_once 'SQL/SearchParams.php';
		
		// search
		$search = new SearchParams();
		$search->setFormSubmitField(@$_GET['search']);
		
		//$search->setFullTextSearch();
		//$search->setParams(@$_GET['phrase'], 'phrase', array('phrase', 'definition'));
		
		
		// setParams($value, $form_field, $sql_field, $group = false, $operator = false) {
		$val = $search->parseSeachString(@$_GET['search_str']);
		$val = $search->setLikeSql($val, true);
		$search->setParams($val, 'search_str', 'question', false);
		$search->setParams($val, 'search_str1', 'answer', false, 'OR');
		
		return $search->getSQL();
	}
*/
	
	
	
	function getFullSearchParams() {
		
 		$str = mysql_escape_string(stripslashes($_GET['search_str']));
		$word = 'score';		
		$sql = "MATCH (e.title, e.body) AGAINST ('$str')";
		$sql1 = "MATCH (e.title, e.body) AGAINST ('$str' IN BOOLEAN MODE)";
			
		$s['select'] = $sql . ' AS ' . $word;
		$s['where'] = $sql1;
		$s['order'] = "ORDER BY $word DESC";
		
		return $s;
	}	
	
	
	// return true if article is private and we use it
	function isEntryPrivate($article_id) {
		$sql = "SELECT private FROM {$this->tbl->entry} WHERE id = ?";
		$result =& $this->db->Execute($sql, $article_id) or die(db_error($sql));
		return $result->Fields('private');
	}
	
	
	// $what = both, article or category
	function isPrivate($what, $id) {
		if($what == 'both') {
			
		}
	}
	
	
	// GLOSSARY // ------------------------------	
	
	function &getGlossaryItems() {
		$sql = "SELECT g.id, g.phrase FROM {$this->tbl->glossary} g";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();
	}
	
	
	function getGlossaryDefinitions($ids = false) {
	
		$param = ($ids) ? "id IN ($ids)" : "1";
	
		$sql = "SELECT g.phrase, g.definition, g.display_once
		FROM {$this->tbl->glossary} g
		WHERE {$param}
		ORDER BY g.phrase";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->GetAssoc();		
	}
	
	
	function isGlossary() {
		$sql = "SELECT COUNT(*) AS num FROM {$this->tbl->glossary} g";
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->Fields('num');
	}	
	
	
	// COMMENTS // ------------------------------	
	
	function getCommentListSql($entry_id) {
		$sql = "
		SELECT 
			c.*, 
			UNIX_TIMESTAMP(date_posted) AS ts,
			IFNULL(u.username, c.name) AS name,
			IF(u.email IS NOT NULL, '', c.email) AS email
		FROM 
			{$this->tbl->comment} c
		LEFT JOIN {$this->tbl->user} u ON u.id = c.user_id
		WHERE 1 
			AND c.entry_id = %d
			AND c.active = 1
		ORDER BY date_posted DESC";
		return sprintf($sql, $entry_id);
	}
	
	
	function getCommentListCountSql($entry_id) {
		$sql = "SELECT COUNT(*) AS num
		FROM {$this->tbl->comment} c
		WHERE c.entry_id = %d
		AND c.active = 1
		ORDER BY date_posted DESC";
		return sprintf($sql, $entry_id);
	}
	
	
	function getCommentList($entry_id, $limit, $offset) {
		$sql = $this->getCommentListSql($entry_id);
		$result =& $this->db->SelectLimit($sql, $limit, $offset) or die(db_error($sql));
		return $result->GetArray();
	}
	
	
	// just to know if any comment exists
	function isComments($entry_id) {
		$sql = $this->getCommentListCountSql($entry_id);
		$result =& $this->db->Execute($sql) or die(db_error($sql));
		return $result->Fields('num');
	}
	
	
	// ACTIONS // ------------------------------
	
	function addComment($entry_id, $data) {
		
		$active = $this->getSetting('comment_status');
		$user_id = ($this->user_id) ? $this->user_id : 'NULL';
		
		$sql = "INSERT {$this->tbl->comment} SET
		entry_id = '$entry_id',
		user_id = $user_id,
		date_posted = NOW(),
		name    = '$data[name]', 
		email   = '$data[email]',
		comment = '$data[comment]',
		active = '$active'";
		
		$this->db->Execute($sql) or die(db_error($sql));
	}

	
	function addUserEntry($data) {
	
		$user_id = ($this->user_id) ? $this->user_id : 'NULL';
	
		$sql = "INSERT {$this->tbl->entry_by_user} SET		
		user_id = $user_id,
		title = '$data[message]',
		email = '$data[email]',
		name = '$data[name]',
		category_id = '$data[category_id]',
		date_posted = NOW(),
		answered = 0";
		$this->db->Execute($sql) or die(db_error($sql));
	}	


	function addVote($entry_id, $rate) {
		
		$action = ($this->isEntryVoted($entry_id)) ? 'UPDATE' : 'INSERT';
		
		if($action == 'UPDATE') {
			$sql = "UPDATE {$this->tbl->rating} SET
			votes    = votes+1, 
			rate     = rate+{$rate}
			WHERE entry_id = {$entry_id}";
		
		} else {
			$sql = "INSERT {$this->tbl->rating} SET
			entry_id = {$entry_id}, 
			votes    = 1, 
			rate     = {$rate}";
		}
		
		$this->db->Execute($sql) or die(db_error($sql));
	}
	
	
	function addView($entry_id) {
		$sql = "UPDATE {$this->tbl->entry} SET 
		hits = hits+1, 
		date_updated=date_updated 
		WHERE id = ?";
		$this->db->Execute($sql, $entry_id) or die(db_error($sql));
	}	
	
	
	// if article was already voted 
	function isEntryVoted($entry_id) {
		$sql = "SELECT entry_id FROM {$this->tbl->rating} WHERE	entry_id = ?";
		$result = &$this->db->Execute($sql, $entry_id) or die(db_error($sql));
		return $result->Fields('entry_id');
	}
	
	
	// check if user already vore
	function isUserVoted($entry_id) {
		$ret = false;

		if(isset($_SESSION[$this->session_vote_name])) {
			if(in_array($entry_id, $_SESSION[$this->session_vote_name])) {
				$ret = true;
			}
		}
		
		return $ret;
	}
	
	
	function setUserVoted($entry_id) {
		$_SESSION[$this->session_vote_name][] = $entry_id;
	}
	
	
	function setUserViewed($entry_id) {
		$_SESSION[$this->session_view_name][] = $entry_id;
	}
	
	
	// check if user already view
	function isUserViewed($entry_id) {
		$ret = false;
		
		if(isset($_SESSION[$this->session_view_name])) {
			if(in_array($entry_id, $_SESSION[$this->session_view_name])) {
				$ret = true;
			}
		}
		
		return $ret;
	}
	
	
	function isFlood($sec = 30) {
		$ret = false;
		
		if(!empty($_SESSION['flood_'])) {
			
			$time_to_check = $_SESSION['flood_']+$sec;
			if($time_to_check > time()) {
				$ret = $time_to_check - time();
			} else {
				$_SESSION['flood_'] = time();
			}
			
		} else {
			$_SESSION['flood_'] = time();
		}
		
		return $ret;
	}
	

/*
	function isFlood($sec = 30) {
		require_once 'eleontev/Util/Antiflood.php';
		
		$af = new Antiflood(APP_CACHE_DIR);
		$af->af_prefix = "flood_";      
		$af->af_rules = array($sec => 1); // no more 1 message per 30 seconds
		return $af->getTimeout();			
	}
*/
	
	
	// MAIL // ---------------------------------
	
	
/*
	function getSupportEmail() {
		
		$email = $this->setting['support_email'];
		if(!$email) {
			
			$sql = "SELECT email FROM {$this->tbl->entry} WHERE id = ?";
			$result =& $this->db->Execute($sql, $article_id) or die(db_error($sql));
			return $result->Fields('email');
		}
		
	}
*/
	
	function sendEntryNotification($vars, $view) {
		
		$m = new KBClientMailer($view);
		return $m->sendEntryNotification($vars, $this->getSetting('support_email'));
	}	
	
	
	function sendConfirmRegistration($vars, $view) {
		
		$view_key = $view->view_key;
		$link = sprintf("%s?%s=confirm&ec=%s", APP_CLIENT_PATH, $view_key, md5($vars['email']));
		$vars['confirm_link'] = $link;
		
		$m = new KBClientMailer($view);
		return $m->sendConfirmRegistration($vars);
	}	
		
	
	function sendToFriend($vars, $view) {
		
		$m = new KBClientMailer($view);
		return $m->sendToFriend($vars);		
	}
	
	
	function sendNewPassword($password, $user) {
		
		require_once 'core/app/AppMailSender.php';
		return AppMailSender::generatedPassword($password, APP_CLIENT_PATH, $user);
	}
}
?>