<?php
// +---------------------------------------------------------------------------+
// | This file is part of the KnowledgebasePublisher package                   |
// | KnowledgebasePublisher - web based knowledgebase publisher tool           |
// |                                                                           |
// | Author:  Evgeny Leontev <eleontev@gmail.com>                              |
// | Copyright (c) 2005 Evgeny Leontev                                         |
// |                                                                           |
// | For the full copyright and license information, please view the LICENSE   |
// | file that was distributed with this source code.                          |
// +---------------------------------------------------------------------------+

require_once 'eleontev/Util/CryptEmail.php';
require_once APP_MODULE_DIR . 'knowledgebase/comment/inc/KBCommentView_helper.php';


class KBClientView_entry extends KBClientView_common
{

	function &execute(&$manager) {
		
		$row = &$manager->getEntryById($this->entry_id, $this->category_id);
		if(!$row) { return; }
		
		$this->meta_title = $row['title'];
		$title = ($this->controller->getSetting('show_title_nav')) ? $row['title'] : '';
		
		$data = $this->getTop($manager, $title);
		$data .= $this->getEntry($manager, $row);
		$data .= $this->getCommentList($manager);
		
		if($this->isCommentable($manager, $row['commentable'])) {
			if(!$manager->is_registered && $manager->getSetting('allow_comments') == 2) {
			
			// display form if at least one comment exists
			} elseif($manager->isComments($this->entry_id)) {
				$data .= $this->getCommentForm($manager);		
			}
		}
		
		$data .= $this->getEntryListRelated($manager);	
		
		return $data;		
	}
	
	
	function getEntry(&$manager, &$row)  {
		
		$this->_parseGlossaryItems($row['body'], $manager);
		
		$tpl = &new tplTemplatez($this->template_dir . 'article.html');
		
		$tpl->tplAssign('msg', $this->getActionMsg());
		$tpl->tplAssign('rating', $this->_getRating($row['rating']));
		$tpl->tplAssign('updated_date', $this->getFormatedDate($row['ts_updated']));
		$tpl->tplAssign('article_block', $this->getEntryBlock($row, $manager));
		$tpl->tplAssign('base_href', $this->conf['client_home_path']);
		
		
		$attachment = $manager->getAttachmentList($this->entry_id);
		if($attachment) {
		
			$tpl->tplSetNeededGlobal('attachment');
			
			foreach($attachment as $k => $v) {
				
				$a['attachment_link'] = '';
				$a['attachment_title'] = ($v['title']) ? $v['title'] : $v['filename'];
 				$tpl->tplParse($a, 'attachment_row');
			}
		}
		
		
		
		$tpl->tplParse(array_merge($row, $this->msg));
		return $tpl->tplPrint(1);
	}
	
	
	function getEntryBlock($data, $manager) { 
		
		$tpl = &new tplTemplatez($this->template_dir . 'article_block.html');
		
		$tpl->tplAssign('article_id', $data['id']);
		$tpl->tplAssign('category_id', $data['category_id']);	
		$tpl->tplAssign('print_link', $this->_getLink('print', $data['category_id'], $data['id']));
		
		
		if($manager->getSetting('show_hits')) {
			$tpl->tplSetNeededGlobal('show_hits');
		}
		
		if($manager->getSetting('show_send_link')) {
			$tpl->tplSetNeededGlobal('send_link');
			$tpl->tplAssign('send_link', $this->_getLink('send', $data['category_id'], $data['id']));		
		}		
			
		if($this->isCommentable($manager, $data['commentable'])) {
			$tpl->tplSetNeededGlobal('commentable');
			$tpl->tplAssign('comment_link', $this->_getLink('comment', $data['category_id'], $data['id']));		
		}
		
		
		if($this->isRatingable($manager, $data['ratingable'])) {
			
			$tpl->tplSetNeededGlobal('ratingable');
			
			if($manager->isUserVoted($data['id']) === false) {
				$select = new FormSelect();
				$select->select_tag = false;
				$select->setRange($this->_getRateRange(), array(0=>$this->msg['add_rate_msg']));
				$tpl->tplAssign('rating_select', $select->select());
				$tpl->tplSetNeeded('/vote_form');
			
			} else {
			
				$tpl->tplAssign('rating', $this->_getRating($data['rating']));
				$tpl->tplSetNeeded('/show_rating');
			}
			
			
			$tpl->tplAssign('form_action', $this->_getLink('entry', $data['category_id'], $data['id']));
			$tpl->tplAssign('votes', ($data['votes']) ? $data['votes'] : 0);		
		}		
		
		
		$tpl->tplParse(array_merge($data, $this->msg));
		return $tpl->tplPrint(1);
	}
	
	
	function getEntryListRelated(&$manager) {
		
		$rows = &$manager->getEntriesRelated($this->category_id, $this->entry_id);
		if(!$rows) { return; }
		
		$tpl = &new tplTemplatez($this->template_dir . 'article_list_related.html');
		foreach($rows  as $k => $row) {
			
			$tpl->tplAssign('entry_link', $this->_getLink('entry', $this->category_id, $row['id']));
			$tpl->tplParse($row, 'row');
		}
		
		$tpl->tplAssign('entry_title_msg', $this->msg['entry_related_title_msg']);
		$tpl->tplParse();
		return $tpl->tplPrint(1);
	}
	
		
	
	function getCommentList(&$manager) {
		
		$tpl = &new tplTemplatez($this->template_dir . 'comment_list.html');
		
		$bp = $this->pageByPage(20, $manager->getCommentListCountSql($this->entry_id));
		$tpl->tplAssign('page_by_page', $bp->shortNavigate());
		
		$rows = &$manager->getCommentList($this->entry_id, $bp->limit, $bp->offset);
		if(!$rows) { return; }
		
		
		$parser = &KBCommentView_helper::getBBCodeObj();
		
		foreach($rows as $k => $row) {
			
			if($row['email']) {
				$tpl->tplSetNeeded('row/if_email');
				$tpl->tplAssign('mailto_link', CryptEmail::getMailto($row['email']));
			} elseif($row['name']) {
				$tpl->tplSetNeeded('row/if_name');
			}
			
			$tpl->tplAssign('comment', $parser->qparse(nl2br($row['comment'])));		
			$tpl->tplAssign('formated_date', $this->getFormatedDate($row['ts']));
			$tpl->tplParse($row, 'row');
		}
		
		
		$tpl->tplAssign('js_script', CryptEmail::getMailtolJs(false));
		$tpl->tplAssign($this->msg);
		$tpl->tplParse();
		return $tpl->tplPrint(1);
	}	
	
	
	function getCommentForm($manager) {
		
		$tpl = &new tplTemplatez($this->template_dir . 'comment_form.html');
		
		$tpl->tplAssign('action_link', $this->_getLink('comment', $this->category_id, $this->entry_id));
		
		if(!$manager->is_registered) {
			$tpl->tplSetNeeded('/not_registered');
		}
		
		$tpl->tplAssign($this->msg);
		$tpl->tplAssign($this->getFormData());	
		
		$tpl->tplParse();
		return $tpl->tplPrint(1);
	}
}
?>